@@ -72,19 +72,19 @@ def load_key(algorithm, key_data):
 
 
 class RS64Key(RSKey):
-    SIZE = 64
+    DIGESTSIZE = 256
     HASHNAME = "sha256"
     HASHMOD = hashlib.sha256
 
 
 class RS128Key(RSKey):
-    SIZE = 128
+    DIGESTSIZE = 320
     HASHNAME = "sha256"
     HASHMOD = hashlib.sha256
 
 
 class RS256Key(RSKey):
-    SIZE = 256
+    DIGESTSIZE = 512
     HASHNAME = "sha256"
     HASHMOD = hashlib.sha256
 
